from __future__ import nested_scopes
from common import *
import Parser
import re

class Optimizer:
    def optimize(self, tree):
        return tree

class WhitespaceCompressor(Optimizer):
    def optimize(self, tree):
        treelist = tree.walk()
        for node in treelist:
            if node.__class__ == Parser.TextToken:
                if not node.has("TT"):
                    node.data = self.compressWS(node.data)
        return tree

    def compressWS(self, text):
        text = re.sub("[\s\n]+", " ", text)
        return text


# Must not be used until after Overlap Inversion or it may detect
# valid constructs as redundancies
class RedundancyEliminator(Optimizer):
    def __init__(self):
        self.taglist = []
        self.taglist += smlngtags
        # can't remove EM tags, because they aren't redundant: they
        # change/invert emphasis levels when nested
        self.taglist.remove("EM")

    def optimize(self, tree):
        treelist = tree.walk()
        for node in treelist:
            if node.__class__ == Parser.TextToken:
                redundant = self.redundancyFound(node)
                if redundant:
                    self.clean(node, redundant)
        return tree

    def redundancyFound(self, node):
##        moved getProp to common.py
##        def getProp(node):
##            return node.data
        proplist = map(getProp,node.propList())
        scanlist = []
        for prop in proplist:
            if prop not in scanlist:
                scanlist.append(prop)
        if len(scanlist) < len(proplist):
            for i in scanlist:
                proplist.remove(i)
            return proplist
        return None

    def clean(self, node, redundancy):
        """ Call this when <node> has a definite <redundancy> in its parent list. """
        pass

class EMEliminator(Optimizer):
    def optimize(self, tree):
        treelist = tree.walk()
        for node in treelist:
            if getProp(node) == 'EM':
                proplist = map(getProp, node.propList())
                if 'S' in proplist:
##                    for child in node.children:
##                        node.parent.addChild(child)
                    place = node.parent.children.index(node)
                    node.parent.children = node.parent.children[:place] + node.children + node.parent.children[place+1:]
                    del node

        return tree



class OverlapInversion(Optimizer):
    def optimize(self, tree):
        return tree

class WhiteSpaceSimplification(Optimizer):
    def optimize(self, tree):
        t = tree
        prev = None
        next = None
        while 1:
            t = t.next
            if not t: break
            prev = t.prev
            next = t.next
            if prev and t.cache == prev.cache: print "LEFT"
            if next and t.cache == next.cache: print "RIGHT"
        return tree

class PrepTree(Optimizer):
    stack = []
    alike = {}
    def gather(self, token):
        #Cache all props
        if isinstance(token, Parser.TextToken):
            token.propList()
            if self.stack:
                prev = self.stack[-1]
                prev.next = token
                token.prev = prev
                token.next = None
            self.stack.append(token)

    def optimize(self, tree):
        self.stack.append(tree)
        tree.enum(self.gather)

        tree.next.prev = None #Break the back link, cause root is not a text node

##        t = tree
##        while 1:
##            t = t.next
##            self.alike.setdefault(t.cache, []).append(t)
##        print "ALIKE", self.alike

        return tree

