import re
from common import *
from PropertyRecord import Properties

validTags = raw_numbers + baseTags

tagPattern = re.compile("<(?P<close>\/)?(?P<tag>[\w\d]+)>")



class Token(Node):
  data = None
  parent = None
  props = None
  cache = None
  def __init__(self, tag):
    Node.__init__(self)
    self.data = tag

  def dump(self, output):
    output.write(str(self))

  def __str__(self):
    res = ["<%s>" % self.data]
    for child in self.children:
      res.append("%s" % child)
    res.append("</%s>" % self.data)
    return ''.join(res)

  def has(self, prop):
    """ Do I possess the given property? i.e. is it in my parent chain?
        return 1 on true
        return None on false
    """
    if self.startOf(prop):
      return 1
    elif self.parent is None:
      return None
    else:
      return self.parent.has(prop)

  def startOf(self, prop):
    """ Am I a <prop> node?
        return 1 on yes
        return None on no
    """
    if self.data == prop:
      return 1
    return None

  def enum(self, callable):
    if self.children is not None and len(self.children) > 0:
      for child in self.children:
        callable(child)
        child.enum(callable)
      callable(self)

  def walk(self, walklist=[]):
    if self.children is not None and len(self.children) > 0:
      # let children add to the list first
      for child in self.children:
        walklist = walklist + child.walk()
    #walklist.append(self)
    #walklist = walklist + [self]
    return walklist + [self]

  def parentList(self, plist=[]):
    """ list of a node's parents, plus itself at the end """
    if self.parent is not None:
      plist = plist + self.parent.parentList(plist)
    #plist.append(self)
    return plist + [self]

  def propList(self):
    #This can never change or we lose eq.
    if self.props == None:
      self.props = []
      self.cache = {}
##      if self.__class__ != TextToken:
##        raise TypeError, "Not a text node."
      for node in self.parentList()[:-1]:
        if node.data in smlngtags:
          self.props.append(node)
          self.cache[node.data] = 1
    return self.props


class TextToken(Token):
  def __str__(self):
    return self.data or ''



class TokenRoot(Token):
  def __str__(self):
    res = []
    for child in self.children:
      res.append(str(child))
    return ''.join(res)

class Parser:
  """Build a parse tree"""

  def parse(self, input):
    tree  = TokenRoot(SPECIAL_TAG)
    stack = [tree]

    head = 0
    while 1:
      top = stack[-1]
      match = tagPattern.search(input, head)
      if match:
         # Add leading text token
         text = input[head:match.start()]
         if text:
           top.addChild(TextToken(text))
         # Advance the head
         head = match.end()

         if match.group("close"):
           #close the scope
           assert top.data == match.group("tag"), """Improper nesting, expected %s, got %s""" %( stack[-1].data, match.group("tag"))
           stack.pop()
         else:
           #open a new scope
           t = Token(match.group("tag"))
           assert (t.data in validTags), "Invalid tag in document: %s"% t.data
           if t:
             top.addChild(t)
           stack.append(t)
      else:
        text = input[head:]
        if text:
          top.addChild(TextToken(text))
        break

    return tree

