import re
from common import *

class SmlngWriter:
  def _start(self, end=None):
    if end: end = '/'
    else: end = ''
    return "<%s%s>" %(end, self.name)

  def _text(self):
    # are we in a TT context
    text = self['text']
    if not self.inTT():
      text = self._reduceWS(text)
    return text

  def _reduceWS(self, text):
    if not text: return ''
    new = re.sub("[\s\n]+", " ", text)
    #if len(text) != len(new):
    #  print "reduced %r to %r" % (text, new)
    return new

  def _end(self):
    return "</%s>" % (self.name)

  def emit(self, output, msg):
    output.write(msg)

  def dump(self, output):
    if self.name==TEXT:
      #print "TEXT", repr(self._text())
      self.emit(output, self._text())
      return

    if len(self.children):
      self.emit(output, "%s%s" %(self._start(), self._text()))
      for child in self.children:
        child.dump(output)
      self.emit(output, self._end())
    else:
      #No children
      self.emit(output, "%s%s%s" %(self._start(), self._text(), self._end()))

class PropertyRecord(SmlngWriter):
  def __init__(self, name, parent=None):
    self.__dict__['name']   = name
    self.__dict__['parent'] = parent
    self.__dict__['data']   = {}
    self.__dict__['children'] = []

    if parent:
      parent.addChild(self)

  def __cmp__(self, other):
    return self.data == other.data

  def __repr__(self):
    return "<Prop(%s) %s>" % (len(self.children), self.data)

  def setProp(self, name, value):
    self[name] = value

  def setData(self, text):
    self['text'] = text

  def addChild(self, child):
    self.children.append(child)

  def __setitem__(self, key, value):
    #If there is a method with the prescribed name
    # invoke that
    methName = "_set_%s" % key

    if hasattr(self, methName):
      method = getattr(self, methName)
      method(value)
      #warn("Called %s" % method)
    else:
      self.data[key] = value

  def __getitem__(self, key):
    prop = None
    try:
      prop = self.data[key]
    except:
      if self.parent is not None:
        try:
          prop = self.parent[key]
        except:
          prop = None
    return prop

  def _set_EM(self, value):
    current = self["EM"]
    if not current:
      current = -1
    self["EM"] *= -1

  def _set_U(self, value):
    current = self["u"]
    if current and  (current + value) < 3:
      self['u'] = current + 1
    else:
      self['u'] = 1


  def inTT(self):
    return self['TT'] >= 1

  def _set_zero(self, value):  self['size'] = 0
  def _set_one(self, value):   self['size'] = 1
  def _set_two(self, value):   self['size'] = 2
  def _set_three(self, value): self['size'] = 3
  def _set_four(self, value) : self['size'] = 4
  def _set_five(self, value):  self['size'] = 5
  def _set_six(self, value):   self['size'] = 6
  def _set_seven(self, value): self['size'] = 7
  def _set_eight(self, value): self['size'] = 8
  def _set_nine(self, value):  self['size'] = 9



class PropRoot(PropertyRecord):
  def emit(self, output, msg):
    pass



class Properties(Node):
  def __init__(self, parent):
    Node.__init__(self)
    self.parent = parent
    self.B  = 0
    self.EM = -1
    self.I  = 0
    self.S  = 0
    self.TT = 0
    self.U  = 0
    self.size  = 0
    self.color = None
    self.text = ''

  def __str__(self):
    res = ["<%s>" % self.data]
    for child in self.children:
      res.append("%s" % child)
    res.append("</%s>" % self.data)
    return ''.join(res)


  def set(self, key):
    name = "set%s" % key
    if hasattr(self, name):
      method = getattr(self, name)
      method()
    else:
      print "UNKNOWN ATTR(%s)" % key

  def setText(self, text):
    if not self.TT:
      text = self.reduceWS(text)
    self.text = text

  def reduceWS(self, text):
    text = re.sub("[\s\n]+", " ", text)
    return text

  def __getattr__(self, key):
    if key.startswith("__") and key.endswith("__"):
      return self.__dict__[key]

    prop = None
    try:
      getattr(self, key)
    except:
      if self.parent is not None:
        try:
          prop = getattr(self.parent, key)
        except:
          prop = None
    return 0

  def setB(self): self.B = 1

  def setEM(self): self.em *= -1

  def setI(self): self.I = 1

  def setS(self):
    self.S = 1
    self.EM = -1

  def setTT(self):
    self.TT = 1

  def setU(self):
    current = self.U
    self.U = max(current +1, 3)

  def set0(self): self.size = 0
  def set1(self): self.size = 1
  def set2(self): self.size = 2
  def set3(self): self.size = 3
  def set4(self): self.size = 4
  def set5(self): self.size = 5
  def set6(self): self.size = 6
  def set7(self): self.size = 7
  def set8(self): self.size = 8
  def set9(self): self.size = 9

  def setr(self): self.color = "r"
  def setg(self): self.color = "g"
  def setb(self): self.color = "b"
  def setc(self): self.color = "c"
  def setm(self): self.color = "m"
  def sety(self): self.color = "y"
  def setk(self): self.color = "k"
  def setw(self): self.color = "w"

  def setPL(self):
    self.S  = 0
    self.EM = -1
    self.I  = 0
    self.B  = 0
    self.TT = 0
    self.U  = 0

