#!/usr/bin/env python
# python 2.1+
from __future__ import nested_scopes
import sys, random
from smlng import *
import Handler
from PropertyRecord import *

# by default, half the time a tag will be generated
# and each tag has an equal probability of being produced
# i.e. the sum of the values is the probability that a tag
# will be generated at all: 25 * 0.02 = 0.5 or half the time
default_probabilities = {
    'B': 0.02,
    'EM': 0.02,
    'I': 0.02,
    'PL': 0.02,
    'S': 0.02,
    'TT': 0.02,
    'U': 0.02,
    '0': 0.02,
    '1': 0.02,
    '2': 0.02,
    '3': 0.02,
    '4': 0.02,
    '5': 0.02,
    '6': 0.02,
    '7': 0.02,
    '8': 0.02,
    '9': 0.02,
    'r': 0.02,
    'g': 0.02,
    'b': 0.02,
    'c': 0.02,
    'm': 0.02,
    'y': 0.02,
    'k': 0.02,
    'w': 0.02
    }

# some gibberish to put inside the tags
# each word/phrase has a 0.04 percent chance of being chosen
words = ['foo', 'bar', 'gleep', 'butternuts', 'bite me fanboy',
         'bumhug', 'cornholio', 'sigma', 'lambda', 'upsilon',
         'tau', 'beer', 'cornnuts', 'skittles', 'pretzels',
         'pizza', 'wings', 'cheezy poofs', 'semiconductor physics',
         'diablo', 'concurrent', 'version', 'system',
         'python is the programming tool of choice for discriminating hackers']
         
def generate(out, length=100, roots=1, maxdepth=5, probabilities=None):
    if probabilities is None:
        probabilities = default_probabilities

    tree = Tree(roots, maxdepth, probabilities)

    while tree.doclength() < length:
        tree.event(words)
    
    tree.stack[0].dump(out)

def openTag(tagname):
    return "<%s>" % tagname

def closeTag(tagname):
    return "</%s>" % tagname

class Tree(Handler.SmlngHandler, SmlngWriter):
    def __init__(self, roots, maxdepth, probs, output=sys.stderr):
        self.roots = roots
        self.rootct = 0
        self.maxdepth = maxdepth
        self.probs = probs
        self.probtag = self.tagprob()
        self.stack = [PropertyRecord("SMLNG")]
        self.length = 0

    def doclength(self):
        return self.length

    def tagprob(self):
        probtot = 0.0
        for val in self.probs.values():
            probtot = probtot + val
        if probtot > 1.0:
            raise ValueError, "Probability of generating a tag is greater than 100 percent."
        return probtot

    def _mapElement(self, name):
        # don't need to map numbers to names for this
        return name
    def _checkTag(self, name):
        # don't need to play xml games
        pass

    def notify(self, event):
        if event.__class__ == StartElementEvent and len(self.stack) < self.maxdepth:
            #print "notified of tag: %s" % event.getName()
            self.length = self.length + 3
            self.startElement(event.getName(), None)
        elif event.__class__ == StartElementEvent and len(self.stack) >= self.maxdepth:
            self.length = self.length + 3
            self.notify(EndElementEvent(self.stack[-1].name)) # too deep, gotta close one
        elif event.__class__ == EndElementEvent and len(self.stack) == 1:
            # at root, can't end right now
            pass
        elif event.__class__ == EndElementEvent and len(self.stack) > 1:
            self.length = self.length + 3
            self.endElement(self.stack[-1].name)
        elif event.__class__ == EndElementEvent and self.rootct <= self.roots:
            self.length = self.length + 3
            self.endElement(self.stack[-1].name)
        elif event.__class__ == EndElementEvent and self.rootct > self.roots and len(self.stack) <= 1:
            # can't end an element here, we'll violate the specified root count
            raise ValueError, "Violated root count"
        elif event.__class__ == CharacterEvent:
            self.length = self.length + len(event.getChars())
            self.characters(event.getChars())

##        if event.__class__ == CharacterEvent:
##            self.length = self.length + len(event.getChars())
##        else:
##            self.length = self.length + 3

    def event(self, words):
        if random.random() <= self.probtag:
            if random.random() <= 0.5:
                tag = self.gettag()
                #print "generated tag: %s" % tag
                self.notify(StartElementEvent(tag))
            else:
                #print "ending tag"
                self.notify(EndElementEvent('x'))
        else:
            #print "character event"
            self.notify(CharacterEvent(random.choice(words)))

    def gettag(self):
        # this will use probabilities later
        return random.choice(self.probs.keys())
    

class Event:
    def getName(self):
        return self.name

class StartElementEvent(Event):
    def __init__(self, name):
        self.name = name

class EndElementEvent(Event):
    def __init__(self, name):
        self.name = name

class CharacterEvent(Event):
    def __init__(self, chars):
        self.name = 'characters'
        self.chars = chars

    def getChars(self):
        return self.chars

def usage():
    print 'USAGE: %s [-h] [-o outputFilename] [-r roots] [-m maxlength] [-d maxdepth] [-p probsFile]' % os.path.basename(sys.argv[0])
    sys.exit(0)

if __name__ == "__main__":
    import getopt
    opts, args = getopt.getopt(sys.argv[1:], 'ho:r:m:d:p:', ['--help',
                                                           '--output',
                                                           '--roots',
                                                           '--maxlen',
                                                           '--maxdepth',
                                                           '--probs'])
    opt_output   = None
    opt_roots    = 1
    opt_maxlen   = 500
    opt_maxdepth = 15
    opt_probs    = None
    
    for opt, arg in opts:
        if opt in ('-o', 'output'):
            opt_output = arg
        elif opt in ('-h', 'help'):
            usage()
        elif opt in ('r','roots'):
            opt_roots = int(arg)
        elif opt in ('m','maxlen'):
            opt_maxlen = int(arg)
        elif opt in ('p', 'probs'):
            opt_probs = arg
        elif opt in ('d', 'maxdepth'):
            opt_maxdepth = int(arg)

    if opt_output is None:
        opt_output = sys.stdout
    else:
        opt_output = open(opt_output, "w")

    if opt_probs is not None:
        f = open(opt_probs, "r")
        d = f.read()
        opt_probs = eval(d)

        generate(opt_output, length=opt_maxlen, roots=opt_roots, maxdepth=opt_maxdepth, probabilities=opt_probs)
    else:
        generate(opt_output, length=opt_maxlen, roots=opt_roots, maxdepth=opt_maxdepth)
