#Python 2.1+
from Handler import *
from Optimizers import *
from Parser import *
from __future__ import nested_scopes
from cStringIO import StringIO
from common import *
from types import *
import os, os.path
import re
import sys
import time


class Timer:
  def __init__(self, limit):
    self.start = time.time()
    if limit == -1:
      self.limit = -1
    else:
      self.limit = self.start + limit # -1?
  def remaining(self):
    if self.limit == -1:
      return -1
    return self.limit - time.time()




class SmlOpt:
  """Top level policy"""
  def __init__(self, timelimit):
    self.timer = Timer(timelimit)

  def optimize(self, input):
    """optimize(input steam, output stream)"""
    if type(input) == FileType:
      tree = self.parse(input)
    else:
      tree = self.parseString(input)

    # convert the parse tree to a more expressive
    # prop tree

    pipeline = [PrepTree(),
                EMEliminator(),
                WhitespaceCompressor(),
                RedundancyEliminator(),
                ]

    #Run opt pipeline
    for optimizer in pipeline:
      tree = optimizer.optimize(tree)
      if self.timer.remaining() <= 1: break

    return tree

  def parse(self, input):
    text = input.read()
    return self.parseString(text)

  def parseString(self, input):
    parser = Parser()
    t = parser.parse(input)
    return t


def fetchTimelimit(args):
  try:
    limit = int(os.environ['TLIMIT'])
  except Exception, E:
    warn("Running without timelimit?")
    limit = -1
  return limit


if __name__ == "__main__":
  #Process options
  args = sys.argv
  timelimit = fetchTimelimit(args)
  input     = sys.stdin

  sml = SmlOpt(timelimit)
  tree = sml.optimize(input)
  sys.stdout.write(tree)




