import sys
import unittest
import Parser
import Optimizers
import validate
from common import *

class TestOptimizations(unittest.TestCase):
    def setUp(self):
        self.p = Parser.Parser()

    def _equiv(self, input, tree, show=0):
        output = str(tree)
        if show == 2:
            print >>sys.stderr, repr(output)
        #Validate output
        rc = validate.SMLEquivalenceValidate(input, output, view=show)
        assert(rc == 1), """Not Equiv: %d""" %(rc)

    def _optimized(self, input, tree):
        output = str(tree)
        assert(len(output) <= len(input)), """Didn't actually optimize output"""
    def testGetProp(self):
        input = """<S> foo <EM> bar </EM> </S> <EM> <EM> foo </EM> </EM>."""
        t = self.p.parse(input)
        node = t.children[0].children[1]
        assert(getProp(node) == 'EM')

    def testRedundancyFound(self):
        r = Optimizers.RedundancyEliminator()
        input = """<B><I><B>foo</B></I></B>"""
        t = self.p.parse(input)
        node = t.children[0].children[0].children[0].children[0]
        assert(r.redundancyFound(node))
        input = """<I><B>foo</B></I>"""
        t = self.p.parse(input)
        node = t.children[0].children[0].children[0]
        assert(r.redundancyFound(node) is None)

##    def testRedundancyElimination(self):
##        r = Optimizers.RedundancyEliminator()
##        input = """<B><I><B>foo</B></I></B>"""
##        t = self.p.parse(input)
##        tree = r.optimize(t)
##        self._equiv(input, tree, show=0)
##        self._optimized(input, tree)

    def testWhitespaceCompression(self):
        r = Optimizers.WhitespaceCompressor()
        input = """ foo    <EM> <b>  bar
        </b>       noodle  </EM>"""
        t = self.p.parse(input)
        tree = r.optimize(t)
        self._equiv(input, tree, show=0)
        self._optimized(input, tree)

    def testWhitespaceCompressionPreseve(self):
        r = Optimizers.WhitespaceCompressor()
        input = """ foo    <EM> <b>  bar
        </b> <TT> foo   </TT>      noodle  </EM>"""
        t = self.p.parse(input)
        tree = r.optimize(t)
        self._equiv(input, tree, show=0)
        self._optimized(input, tree)

    def testEMElimination(self):
        r = Optimizers.EMEliminator()
        input = """<S> foo <EM> bar </EM> </S> <EM> <EM> foo </EM> </EM>."""
        t = self.p.parse(input)
        tree = r.optimize(t)
        self._equiv(input, tree, show=0)
        self._optimized(input, tree)


    def testOverlapInvesion(self):
        r = Optimizers.OverlapInversion()
        input    = """<B><I>bla bla</I></B><TT><I><B>foo bar</B></I>truc</TT>"""
        expected = """<B><I>bla bla<TT>foo bar</TT></I></B><TT> truc</TT>"""
        t = self.p.parse(input)
        tree = r.optimize(t)
        self._equiv(input, tree, show=0)
        self._optimized(input, tree)

    def testWhiteSpaceSimplification(self):
        r = Optimizers.WhiteSpaceSimplification()
        input = """<B><I><r>bla bla </r> </I> </B><r><I>barfoo</I></r>"""
        expected = "<I><r><B>bla bla </B>barfoo</r></I>"
        t = self.p.parse(input)
        p = Optimizers.PrepTree()
        tree = p.optimize(t)
        tree = r.optimize(tree)
        self._equiv(input, tree, show=1)
        self._optimized(input, tree)

    def testPrepTree(self):
        r = Optimizers.PrepTree()
        input = """<B><I><r>bla bla </r> </I> </B><r><I>barfoo</I></r>"""
        t = self.p.parse(input)
        tree = r.optimize(t)
        l = []
        t = tree
        while 1:
            t = t.next
            if not t: break
            l.append(t)

        assert [n.data for n in l] == ['bla bla ', ' ', ' ', 'barfoo']
        #print [n.cache for n in l]

if __name__ == "__main__":
    unittest.main()
