import unittest
import smlng
import sys
from cStringIO import StringIO
import validate
import Parser
from common import *

class TestSmlng(unittest.TestCase):
  def setUp(self):
    self.p = Parser.Parser()

  def testParserStatic(self):
    input = """
    <U><r>foo<g>bar</g></r></U><g>xx</g>
    """
    tokens = self.p.parse(input)
    assert str(tokens) == input, """Parser output not eq"""


  def testInvalidTag(self):
    input = """<U><bz>not a tag</bz></U><g>xx</g>"""
    try:
      t = self.p.parse(input)
    except:
      pass
    else:
      self.fail("Didn't detect invalid tag")

  def testImpoperNesting(self):
    input = """<b><I>foo</b></I>"""
    try:
      self.p.parse(input)
    except:
      pass
    else:
      self.fail("Didn't detect improper nesting")

  def testBuildTree(self):
    input = """<b>foo</b><b>bar</b>"""
    t = self.p.parse(input)
    assert(len(t.children) == 2), """Funky tree"""

  def testHas(self):
    input = """foo <b> bar <g> pucker </g> foolio </b>"""
    t = self.p.parse(input)
    node = t.children[1].children[1].children[0]
    assert(node.has('b'))
    node = t.children[1].children[2]
    assert(node.has('g') is None)

  def testStartOf(self):
    input = """foo <b> bar <g> pucker </g> foolio </b>"""
    t = self.p.parse(input)
    node = t.children[1]
    assert(node.startOf('b'))
    node = t.children[1].children[0]
    assert(node.startOf('b') is None)

  def testWalk(self):
    input = """foo <b> bar <g> pucker </g> foolio </b>"""
    t = self.p.parse(input)
    tlist = t.walk()
    print "Length: %d" % len(tlist)
    print [getProp(x) for x in tlist]
    assert(len(tlist) == 7) # 6 nodes plus magic SMLNG node

  def testParentList(self):
    input = """foo <b> bar <g> pucker </g> foolio </b>"""
    t = self.p.parse(input)
    child = t.children[1].children[1].children[0]
    plist = child.parentList()
    assert(len(plist) == 4), "Call one."
    plist = child.parentList()
    assert(len(plist) == 4), "Call two."

  def testPropertyList(self):
    input = """foo <b> bar <g> pucker </g> foolio </b>"""
    t = self.p.parse(input)
    child = t.children[1].children[1].children[0]
    proplist = child.propList()
    
if __name__ == "__main__":
  unittest.main()
