import unittest
import smlng
import sys
from cStringIO import StringIO
import validate


class TestSmlng(unittest.TestCase):

  def _validateOutput(self, tree):
    output = str(tree)
    #Validate output
    rc = validate.SMLSyntaxValidate(output)
    assert(rc == 1)

  def _equiv(self, input, tree, show=0):
    output = str(tree)
    if show == 2:
      print >>sys.stderr, repr(output)
    #Validate output
    rc = validate.SMLEquivalenceValidate(input, output, view=show)
    assert(rc == 1), """Not Equiv: %d""" %(rc)

  def _isOptimized(self, input, tree, verbose=None):

    if verbose:
      leninput = len(input)
      lentree = len(str(tree))
      optpct = float(lentree) / float(leninput)
      print "Input size : %d" % leninput
      print "Output size: %d" % lentree
      print "Reduction  : %f" % optpct
      if verbose == 2:
        print "Input : %s" % input
        print "Output: %s" % str(tree)


  def testHandler(self):
    input = "<r>aaa</r><g>bbb</g><b>ccc</b><g>ddd</g><r>eee</r>"
    sml = smlng.SmlOpt(-1)
    tree = sml.optimize(input)
    self._validateOutput(tree)

  def testWS(self):
    input = "'\n\n'"
    sml = smlng.SmlOpt(-1)
    tree = sml.optimize(input)
    self._validateOutput(tree)
    self._equiv(input, tree, show=0)

  def testProps(self):
    input = "<U><r><U>foo</U></r></U>"
    sml = smlng.SmlOpt(-1)
    tree = sml.optimize(input)
    self._validateOutput(tree)

  def testTree(self):
    input = """<U>
       foo
       <r>more
         <3>large</3>
       </r>
    </U>"""
    sml = smlng.SmlOpt(-1)
    tree = sml.optimize(input)
    self._validateOutput(tree)
    self._equiv(input, tree, show=0)

  def testOptimize(self):
#    f = open("sample/sample.sml")
#   input = f.read()
#    f.close()
    input = """<S> foo
                 <EM> bar </EM>

baz
                         <b>gleep</b>
               </S>"""
    optimizer = smlng.SmlOpt(180)
    tree = optimizer.optimize(input)
    self._validateOutput(tree)
    self._equiv(input, tree, show=1)
    self._isOptimized(input, tree) # ,verbose=2)


if __name__ == "__main__":
  unittest.main()
